@@ -56,18 +56,18 @@ inline Agent<TSeq>::Agent(Agent<TSeq> && p) :
 template<typename TSeq>
 inline Agent<TSeq>::Agent(const Agent<TSeq> & p) :
     model(p.model),
-    neighbors(p.neighbors),
-    neighbors_locations(p.neighbors_locations),
     n_neighbors(p.n_neighbors),
     entities(p.entities),
     entities_locations(p.entities_locations),
-    n_entities(p.n_entities),
-    sampled_agents(0u),
-    sampled_agents_n(0u),
-    sampled_agents_left_n(0u),
-    date_last_build_sample(-99)
+    n_entities(p.n_entities)
 {
 
+    if (n_neighbors > 0u)
+    {
+        neighbors = new std::vector< size_t >(*p.neighbors);
+        neighbors_locations = new std::vector< size_t >(*p.neighbors_locations);
+    }
+
     state = p.state;
     id     = p.id;
     
@@ -101,23 +101,23 @@ inline Agent<TSeq> & Agent<TSeq>::operator=(
 
     model = other_agent.model;
 
-    neighbors = other_agent.neighbors;
-    neighbors_locations = other_agent.neighbors_locations;
     n_neighbors = other_agent.n_neighbors;
+    if (neighbors != nullptr)
+    {
+        delete neighbors;
+        delete neighbors_locations;
+    }
 
+    if (other_agent.n_neighbors > 0u)
+    {
+        neighbors = new std::vector< size_t >(other_agent.n_neighbors);
+        neighbors_locations = new std::vector< size_t >(other_agent.n_neighbors);
+    }
+    
     entities = other_agent.entities;
     entities_locations = other_agent.entities_locations;
     n_entities = other_agent.n_entities;
 
-    sampled_agents.clear();
-    sampled_agents_n = 0;
-    sampled_agents_left_n = 0;
-    date_last_build_sample = -99;
-
-    // neighbors           = other_agent.neighbors;
-    // entities            = other_agent.entities;
-    // entities_locations  = other_agent.entities_locations;
-    // n_entities          = other_agent.n_entities;
     state              = other_agent.state;
     state_prev         = other_agent.state_prev;
     state_last_changed = other_agent.state_last_changed;
@@ -130,7 +130,6 @@ inline Agent<TSeq> & Agent<TSeq>::operator=(
     } else
         virus = nullptr;
     
-    // tools               = other_agent.tools;
     n_tools             = other_agent.n_tools;
     for (size_t i = 0u; i < n_tools; ++i)
     {
@@ -142,6 +141,18 @@ inline Agent<TSeq> & Agent<TSeq>::operator=(
     
 }
 
+template<typename TSeq>
+inline Agent<TSeq>::~Agent()
+{
+
+    if (neighbors != nullptr)
+    {
+        delete neighbors;
+        delete neighbors_locations;
+    }
+
+}
+
 template<typename TSeq>
 inline void Agent<TSeq>::add_tool(
     ToolPtr<TSeq> tool,
@@ -499,10 +510,17 @@ inline void Agent<TSeq>::add_neighbor(
 ) {
     // Can we find the neighbor?
     bool found = false;
-    if (check_source)
+
+    if (neighbors == nullptr)
+    {
+        neighbors = new std::vector< size_t >();
+        neighbors_locations = new std::vector< size_t >();
+    }
+
+    if (check_source && neighbors)
     {
 
-        for (auto & n: neighbors)    
+        for (auto & n: *neighbors)    
             if (static_cast<int>(n) == p.get_id())
             {
                 found = true;
@@ -518,18 +536,18 @@ inline void Agent<TSeq>::add_neighbor(
     if (!found)
     {
 
-        neighbors_locations.push_back(p.get_n_neighbors());
-        neighbors.push_back(p.get_id());
+        neighbors_locations->push_back(p.get_n_neighbors());
+        neighbors->push_back(p.get_id());
         n_neighbors++;
 
     }
 
 
     found = false;
-    if (check_target)
+    if (check_target && p.neighbors)
     {
-
-        for (auto & n: p.neighbors)
+       
+        for (auto & n: *p.neighbors)
             if (static_cast<int>(n) == id)
             {
                 found = true;
@@ -541,8 +559,14 @@ inline void Agent<TSeq>::add_neighbor(
     if (!found)
     {
 
-        p.neighbors_locations.push_back(n_neighbors - 1);
-        p.neighbors.push_back(id);
+        if (p.neighbors == nullptr)
+        {
+            p.neighbors = new std::vector< size_t >();
+            p.neighbors_locations = new std::vector< size_t >();
+        }
+
+        p.neighbors_locations->push_back(n_neighbors - 1);
+        p.neighbors->push_back(id);
         p.n_neighbors++;
         
     }
@@ -558,31 +582,42 @@ inline void Agent<TSeq>::swap_neighbors(
 )
 {
 
+    if (n_this >= n_neighbors)
+        throw std::range_error(
+            "The neighbor you want to swap is out of range. This Agent only has " +
+            std::to_string(n_neighbors) + " neighbors."
+        );
+    if (n_other >= other.n_neighbors)
+        throw std::range_error(
+            "The neighbor you want to swap is out of range. This Agent only has " +
+            std::to_string(other.n_neighbors) + " neighbors."
+        );
+
     // Getting the agents
     auto & pop = model->population;
-    auto & neigh_this  = pop[neighbors[n_this]];
-    auto & neigh_other = pop[other.neighbors[n_other]];
+    auto & neigh_this  = pop[(*neighbors)[n_this]];
+    auto & neigh_other = pop[(*other.neighbors)[n_other]];
 
     // Getting the locations in the neighbors
-    size_t loc_this_in_neigh = neighbors_locations[n_this];
-    size_t loc_other_in_neigh = other.neighbors_locations[n_other];
+    size_t loc_this_in_neigh = (*neighbors_locations)[n_this];
+    size_t loc_other_in_neigh = (*other.neighbors_locations)[n_other];
 
     // Changing ids
-    std::swap(neighbors[n_this], other.neighbors[n_other]);
+    std::swap((*neighbors)[n_this], (*other.neighbors)[n_other]);
 
     if (!model->directed)
     {
         std::swap(
-            neigh_this.neighbors[loc_this_in_neigh],
-            neigh_other.neighbors[loc_other_in_neigh]
+            (*neigh_this.neighbors)[loc_this_in_neigh],
+            (*neigh_other.neighbors)[loc_other_in_neigh]
             );
 
         // Changing the locations
-        std::swap(neighbors_locations[n_this], other.neighbors_locations[n_other]);
+        std::swap((*neighbors_locations)[n_this], (*other.neighbors_locations)[n_other]);
         
         std::swap(
-            neigh_this.neighbors_locations[loc_this_in_neigh],
-            neigh_other.neighbors_locations[loc_other_in_neigh]
+            (*neigh_this.neighbors_locations)[loc_this_in_neigh],
+            (*neigh_other.neighbors_locations)[loc_other_in_neigh]
             );
     }
 
@@ -593,7 +628,7 @@ inline std::vector< Agent<TSeq> *> Agent<TSeq>::get_neighbors()
 {
     std::vector< Agent<TSeq> * > res(n_neighbors, nullptr);
     for (size_t i = 0u; i < n_neighbors; ++i)
-        res[i] = &model->population[neighbors[i]];
+        res[i] = &model->population[(*neighbors)[i]];
 
     return res;
 }
@@ -622,7 +657,7 @@ inline void Agent<TSeq>::change_state(
 }
 
 template<typename TSeq>
-inline const epiworld_fast_uint & Agent<TSeq>::get_state() const {
+inline const unsigned int & Agent<TSeq>::get_state() const {
     return state;
 }
 
@@ -746,7 +781,7 @@ inline void Agent<TSeq>::print(
             static_cast<int>(state),
             virus == nullptr ? std::string("no").c_str() : std::string("yes").c_str(),
             static_cast<int>(n_tools),
-            static_cast<int>(neighbors.size())
+            static_cast<int>(n_neighbors)
         );
     }
     else {
@@ -758,7 +793,7 @@ inline void Agent<TSeq>::print(
         printf_epiworld("  Has virus    : %s\n", virus == nullptr ?
             std::string("no").c_str() : std::string("yes").c_str());
         printf_epiworld("  Tool count   : %i\n", static_cast<int>(n_tools));
-        printf_epiworld("  Neigh. count : %i\n", static_cast<int>(neighbors.size()));
+        printf_epiworld("  Neigh. count : %i\n", static_cast<int>(n_neighbors));
 
         size_t nfeats = model->get_agents_data_ncols();
         if (nfeats > 0)
